@@ -2,6 +2,7 @@ import datetime
 import itertools
 import iso8601
 import pprint
+import re
 
 class _missing(object):
     pass
@@ -18,14 +19,18 @@ class Invalid(Exception):
     The constructor also receives an optional ``msg`` keyword
     argument, defaulting to ``None``.  The ``msg`` argument is a
     freeform field indicating the error circumstance.
+
+    The construct additionally may receive a ``value`` keyword,
+    indicating the value related to the error.
     """
     pos = None
     parent = None
 
-    def __init__(self, node, msg=None):
+    def __init__(self, node, msg=None, value=None):
         Exception.__init__(self, node, msg)
         self.node = node
         self.msg = msg
+        self.value = value
         self.children = []
 
     def add(self, exc, pos=None):
@@ -57,11 +62,11 @@ class Invalid(Exception):
         so, a :exc:`KeyError` is raised.
 
         For example, if the exception upon which ``__setitem__`` is
-        called has a node attribute with children, and that node
-        attribute has children that have the names ``name`` and
-        ``title``, you may successfully call ``__setitem__('name',
-        'Bad name')`` or ``__setitem__('title', 'Bad title')``.  But
-        calling ``__setitem__('wrong', 'whoops')`` will result in a
+        called has a node attribute, and that node attribute has
+        children that have the names ``name`` and ``title``, you may
+        successfully call ``__setitem__('name', 'Bad name')`` or
+        ``__setitem__('title', 'Bad title')``.  But calling
+        ``__setitem__('wrong', 'whoops')`` will result in a
         :exc:`KeyError`.
 
         This method is typically only useful if the ``node`` attribute
@@ -133,28 +138,121 @@ class All(object):
         if msgs:
             raise Invalid(node, msgs)
 
+class Function(object):
+    """ Validator which accepts a function and an optional message;
+    the function is called with the ``value`` during validation.
+
+    If the function returns anything falsy (``None``, ``False``, the
+    empty string, ``0``, an object with a ``__nonzero__`` that returns
+    ``False``, etc) when called during validation, an
+    :exc:`colander.Invalid` exception is raised (validation fails);
+    its msg will be the value of the ``message`` argument passed to
+    this class' constructor.
+
+    If the function returns a stringlike object (a ``str`` or
+    ``unicode`` object) that is *not* the empty string , a
+    :exc:`colander.Invalid` exception is raised using the stringlike
+    value returned from the function as the exeption message
+    (validation fails).
+
+    If the function returns anything *except* a stringlike object
+    object which is truthy (e.g. ``True``, the integer ``1``, an
+    object with a ``__nonzero__`` that returns ``True``, etc), an
+    :exc:`colander.Invalid` exception is *not* raised (validation
+    succeeds).
+
+    The default value for the ``message`` when not provided via the
+    constructor is ``Invalid value``.
+    """
+    def __init__(self, function, message='Invalid value'):
+        self.function = function
+        self.message = message
+
+    def __call__(self, node, value):
+        result = self.function(value)
+        if not result:
+            raise Invalid(node, self.message)
+        if isinstance(result, basestring):
+            raise Invalid(node, result)
+
+class Regex(object):
+    """ Regular expression validator. Initialize it with the string 
+        regular expression ``regex`` that will be compiled and matched 
+        against ``value`` when validator is called. If ``msg`` is 
+        supplied, it will be the error message to be used; otherwise, 
+        defaults to 'String does not match expected pattern'.  
+        
+        When calling, if ``value`` matches the regular expression, 
+        validation succeeds; otherwise, :exc:`colander.Invalid` is 
+        raised with the ``msg`` error message. 
+    """    
+    def __init__(self, regex, msg=None):
+        self.match_object = re.compile(regex)
+        if msg is None:
+            self.msg = "String does not match expected pattern"  
+        else:
+            self.msg = msg
+
+    def __call__(self, node, value):
+        if self.match_object.match(value) is None:
+            raise Invalid(node, self.msg)
+
+class Email(Regex):
+    """ Email address validator. If ``msg`` is supplied, it will be 
+        the error message to be used when raising :exc:`colander.Invalid`; 
+        otherwise, defaults to 'Invalid email address'.  
+    """    
+    def __init__(self, msg=None):
+        if msg is None:
+            msg = "Invalid email address"  
+        super(Email, self).__init__(
+            u'(?i)^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4}$', msg=msg)
+
 class Range(object):
     """ Validator which succeeds if the value it is passed is greater
     or equal to ``min`` and less than or equal to ``max``.  If ``min``
     is not specified, or is specified as ``None``, no lower bound
     exists.  If ``max`` is not specified, or is specified as ``None``,
-    no upper bound exists."""
-    def __init__(self, min=None, max=None):
+    no upper bound exists.
+
+    ``min_err`` is used to form the ``msg`` of the
+    :exc:`colander.Invalid` error when reporting a validation failure
+    caused by a value not meeting the minimum.  If ``min_err`` is
+    specified, it must be a string.  The string may contain the
+    replacement targets ``%(min)s`` and ``%(val)s``, representing the
+    minimum value and the provided value respectively.  If it is not
+    provided, it defaults to ``'%(val)s is less than minimum value
+    %(min)s'``.
+
+    ``max_err`` is used to form the ``msg`` of the
+    :exc:`colander.Invalid` error when reporting a validation failure
+    caused by a value exceeding the maximum.  If ``max_err`` is
+    specified, it must be a string.  The string may contain the
+    replacement targets ``%(max)s`` and ``%(val)s``, representing the
+    maximum value and the provided value respectively.  If it is not
+    provided, it defaults to ``'%(val)s is greater than maximum value
+    %(max)s'``.
+    """
+
+    min_err = '%(val)s is less than minimum value %(min)s'
+    max_err = '%(val)s is greater than maximum value %(max)s'
+
+    def __init__(self, min=None, max=None, min_err=None, max_err=None):
         self.min = min
         self.max = max
+        if min_err is not None:
+            self.min_err = min_err
+        if max_err is not None:
+            self.max_err = max_err
 
     def __call__(self, node, value):
         if self.min is not None:
             if value < self.min:
-                raise Invalid(
-                    node,
-                    '%s is less than minimum value %s' % (value, self.min))
+                raise Invalid(node,self.min_err % {'val':value, 'min':self.min})
 
         if self.max is not None:
             if value > self.max:
-                raise Invalid(
-                    node,
-                    '%s is greater than maximum value %s' % (value, self.max))
+                raise Invalid(node,self.max_err % {'val':value, 'max':self.max})
 
 class Length(object):
     """ Validator which succeeds if the value passed to it has a
@@ -177,7 +275,6 @@ class Length(object):
                     node,
                     'Longer than maximum length %s' % self.max)
 
-
 class OneOf(object):
     """ Validator which succeeds if the value passed to it is one of
     a fixed set of values """
@@ -189,15 +286,7 @@ class OneOf(object):
             raise Invalid(node, '"%s" is not one of %s' % (
                 value, ', '.join(['%s' % x for x in self.values])))
 
-class Type(object):
-    """ Abstract base class for types (only for convenience) """
-    def pserialize(self, node, value):
-        return self.serialize(node, value)
-
-    def pdeserialize(self, node, value):
-        return self.deserialize(node, value)
-
-class Mapping(Type):
+class Mapping(object):
     """ A type which represents a mapping of names to nodes.
 
     The subnodes of the :class:`colander.SchemaNode` that wraps
@@ -209,9 +298,9 @@ class Mapping(Type):
     control the behavior after construction.
 
     unknown
-        ``unknown`` controls the behavior of this type when an unknown
-        key is encountered in the value passed to the ``serialize`` or
-        ``deserialize`` methods of this instance.  The potential
+        ``unknown`` controls the behavior of this type when an
+        unknown key is encountered in the value passed to the
+        ``deserialize`` method of this instance.  The potential
         values of ``unknown`` are:
 
         - ``ignore`` means that keys that are not present in the schema
@@ -222,30 +311,23 @@ class Mapping(Type):
           raised when unknown keys are present during deserialization.
 
         - ``preserve`` will preserve the 'raw' unknown keys and values
-          in the returned data structure.
+          in the returned data structure during deserialization.
 
         Default: ``ignore``.
 
-        Note that the ``pserialize`` and ``pdeserialize`` methods of
-        this type will override this behavior, behaving as if
-        ``unknown`` is ``ignore`` for the duration of those method calls.
-
     partial
         ``partial`` controls the behavior of this type when a
         schema-expected key is missing from the value passed to the
-        ``serialize`` and ``deserialize`` methods of this instance.
-        During serialization and deserialization, when ``partial`` is
-        ``False``, a :exc:`colander.Invalid` exception will be raised
-        if the mapping value does not contain a key specified by the
-        schema node related to this mapping type.  When ``partial`` is
-        ``True``, no exception is raised and a partial mapping will
-        be serialized/deserialized.
+        ``deserialize`` method of this instance.
+
+        During deserialization, when ``partial`` is ``False``, a
+        :exc:`colander.Invalid` exception will be raised if the
+        mapping value does not contain a key specified by the schema
+        node related to this mapping type.  When ``partial`` is
+        ``True``, no exception is raised and a partial mapping will be
+        deserialized.
 
         Default: ``False``.
-
-        Note that the ``pserialize`` and ``pdeserialize`` methods of
-        this type will override this behavior, behaving as if
-        ``partial`` is ``True`` for the duration of those method calls.
     """
 
     def __init__(self, unknown='ignore', partial=False):
@@ -253,7 +335,7 @@ class Mapping(Type):
         self.partial = partial
 
     def _set_unknown(self, value):
-        if not value in ['ignore', 'raise', 'preserve']:
+        if not value in ('ignore', 'raise', 'preserve'):
             raise ValueError(
                 'unknown attribute must be one of "ignore", "raise", '
                 'or "preserve"')
@@ -322,7 +404,8 @@ class Mapping(Type):
             return subnode.serialize(subval)
         def default_callback(subnode):
             return subnode.serialize(subnode.default)
-        return self._impl(node, value, callback, default_callback)
+        return self._impl(node, value, callback, default_callback,
+                          unknown='ignore', partial=True)
 
     def deserialize(self, node, value):
         def callback(subnode, subval):
@@ -331,24 +414,6 @@ class Mapping(Type):
             return subnode.default
         return self._impl(node, value, callback, default_callback)
 
-    def pserialize(self, node, value):
-        def callback(subnode, subval):
-            return subnode.pserialize(subval)
-        def default_callback(subnode):
-            return subnode.serialize(subnode.default)
-        return self._impl(
-            node, value, callback, default_callback, unknown='ignore',
-            partial=True)
-
-    def pdeserialize(self, node, value):
-        def callback(subnode, subval):
-            return subnode.pdeserialize(subval)
-        def default_callback(subnode):
-            return subnode.default
-        return self._impl(
-            node, value, callback, default_callback, unknown='ignore',
-            partial=True)
-
 class Positional(object):
     """
     Marker abstract base class meaning 'this type has children which
@@ -357,7 +422,7 @@ class Positional(object):
     creating a dictionary representation of an error tree.
     """
 
-class Tuple(Type, Positional):
+class Tuple(Positional):
     """ A type which represents a fixed-length sequence of nodes.
 
     The subnodes of the :class:`colander.SchemaNode` that wraps
@@ -411,7 +476,7 @@ class Tuple(Type, Positional):
             return subnode.serialize(subval)
         return self._impl(node, value, callback)
 
-class Sequence(Type, Positional):
+class Sequence(Positional):
     """
     A type which represents a variable-length sequence of nodes,
     all of which must be of the same type.
@@ -519,22 +584,33 @@ Seq = Sequence
 
 default_encoding = 'utf-8'
 
-class String(Type):
-    """ A type representing a Unicode string.  This type constructor
-    accepts a single argument ``encoding``, representing the encoding
-    which should be applied to object serialization.  It defaults to
-    ``utf-8`` if not provided.
+class String(object):
+    """ A type representing a Unicode string.
+
+    This type constructor accepts a single argument ``encoding``,
+    representing the encoding which should be applied to object
+    serialization.  It defaults to ``utf-8`` if not provided.
+
+    Input to ``serialize`` is serialized to a Python ``str`` object,
+    which is encoded in the encoding provided.
+
+    If a string (as opposed to a unicode object) is provided as a
+    value to either the serialize or deserialize method of this type,
+    it must be encoded with the type's encoding; an
+    :exc:`colander.Invalid` error will result if not.
 
     The subnodes of the :class:`colander.SchemaNode` that wraps
     this type are ignored.
     """
     def __init__(self, encoding=None):
+        if encoding is None:
+            encoding = default_encoding
         self.encoding = encoding
     
     def deserialize(self, node, value):
         try:
             if not isinstance(value, unicode):
-                value = unicode(str(value), self.encoding or default_encoding)
+                value = unicode(str(value), self.encoding)
         except Exception, e:
             raise Invalid(node, '%r is not a string: %s' % (value, e))
         if not value:
@@ -545,14 +621,19 @@ class String(Type):
 
     def serialize(self, node, value):
         try:
-            return unicode(value).encode(self.encoding or default_encoding)
+            if isinstance(value, unicode):
+                result = value.encode(self.encoding)
+            else:
+                # do validation here
+                result = unicode(value, self.encoding).encode(self.encoding)
+            return result
         except Exception, e:
             raise Invalid(node,
-                          '%r is cannot be serialized to str: %s' % (value, e))
+                          '%r cannot be serialized to str: %s' % (value, e))
 
 Str = String
 
-class Integer(Type):
+class Integer(object):
     """ A type representing an integer.
 
     The subnodes of the :class:`colander.SchemaNode` that wraps
@@ -576,7 +657,7 @@ class Integer(Type):
 
 Int = Integer
 
-class Float(Type):
+class Float(object):
     """ A type representing a float.
 
     The subnodes of the :class:`colander.SchemaNode` that wraps
@@ -600,7 +681,7 @@ class Float(Type):
 
 Int = Integer
 
-class Boolean(Type):
+class Boolean(object):
     """ A type representing a boolean object.
 
     During deserialization, a value in the set (``false``, ``0``) will
@@ -633,7 +714,7 @@ class Boolean(Type):
 
 Bool = Boolean
 
-class GlobalObject(Type):
+class GlobalObject(object):
     """ A type representing an importable Python object.  This type
     serializes 'global' Python objects (objects which can be imported)
     to dotted Python names.
@@ -740,7 +821,7 @@ class GlobalObject(Type):
         except AttributeError:
             raise Invalid(node, '%r has no __name__' % value)
 
-class DateTime(Type):
+class DateTime(object):
     """ A type representing a Python ``datetime.datetime`` object.
 
     This type serializes python ``datetime.datetime`` objects to a
@@ -757,7 +838,7 @@ class DateTime(Type):
 
     You can adjust the error message reported by this class by
     changing its ``err_template`` attribute in a subclass on an
-    instance of this class.  By default, the ``err_tempalte``
+    instance of this class.  By default, the ``err_template``
     attribute is the string ``%(value)s cannot be parsed as an iso8601
     date: %(exc)s``.  This string is used as the interpolation subject
     of a dictionary composed of ``value`` and ``exc``.  ``value`` and
@@ -806,7 +887,7 @@ class DateTime(Type):
                                                          'exc':e})
         return result
 
-class Date(Type):
+class Date(object):
     """ A type representing a Python ``datetime.date`` object.
 
     This type serializes python ``datetime.date`` objects to a
@@ -848,6 +929,10 @@ class Date(Type):
         return value.isoformat()
 
     def deserialize(self, node, value):
+        if not value:
+            if node.required:
+                raise Invalid(node, 'Required')
+            return node.default
         try:
             result = iso8601.parse_date(value)
             result = result.date()
@@ -942,16 +1027,6 @@ class SchemaNode(object):
         this node."""
         return self.typ.serialize(self, value)
 
-    def pserialize(self, value):
-        """ Partially serialize the value based on the schema
-        represented by this node.  """
-        return self.typ.pserialize(self, value)
-
-    def pdeserialize(self, value):
-        """ Partially deserialize the value based on the schema
-        represented by this node.  """
-        return self.typ.pdeserialize(self, value)
-
     def add(self, node):
         """ Add a subnode to this node. """
         self.children.append(node)
@@ -1007,8 +1082,23 @@ class Schema(object):
 
 MappingSchema = Schema
 
-class SequenceSchema(Schema):
+class SequenceSchema(object):
     schema_type = Sequence
+    node_type = SchemaNode
+    __metaclass__ = _SchemaMeta
+
+    def __new__(cls, *args, **kw):
+        node = object.__new__(cls.node_type)
+        node.name = None
+        node._order = SchemaNode._counter.next()
+        typ = cls.schema_type()
+        node.__init__(typ, *args, **kw)
+        if not len(cls.nodes) == 1:
+            raise Invalid(node,
+                          'Sequence schemas must have exactly one child node')
+        for n in cls.nodes:
+            node.add(n)
+        return node
 
 class TupleSchema(Schema):
     schema_type = Tuple
