@@ -10,16 +10,16 @@ def invalid_exc(func, *arg, **kw):
         raise AssertionError('Invalid not raised') # pragma: no cover
 
 class TestInvalid(unittest.TestCase):
-    def _makeOne(self, node, msg=None, pos=None):
+    def _makeOne(self, node, msg=None, val=None):
         from colander import Invalid
-        exc = Invalid(node, msg)
-        exc.pos = pos
+        exc = Invalid(node, msg, val)
         return exc
 
     def test_ctor(self):
-        exc = self._makeOne(None, 'msg')
+        exc = self._makeOne(None, 'msg', 'val')
         self.assertEqual(exc.node, None)
         self.assertEqual(exc.msg, 'msg')
+        self.assertEqual(exc.value, 'val')
         self.assertEqual(exc.children, [])
 
     def test_add(self):
@@ -70,13 +70,14 @@ class TestInvalid(unittest.TestCase):
         node2 = DummySchemaNode(Positional(), 'node2')
         node3 = DummySchemaNode(Positional(), 'node3')
         node4 = DummySchemaNode(Positional(), 'node4')
-        exc1 = self._makeOne(node1, 'exc1', pos=1)
-        exc2 = self._makeOne(node2, 'exc2', pos=2) 
-        exc3 = self._makeOne(node3, 'exc3', pos=3)
-        exc4 = self._makeOne(node4, 'exc4', pos=4)
-        exc1.add(exc2)
-        exc2.add(exc3)
-        exc1.add(exc4)
+        exc1 = self._makeOne(node1, 'exc1')
+        exc1.pos = 1
+        exc2 = self._makeOne(node2, 'exc2')
+        exc3 = self._makeOne(node3, 'exc3')
+        exc4 = self._makeOne(node4, 'exc4')
+        exc1.add(exc2, 2)
+        exc2.add(exc3, 3)
+        exc1.add(exc4, 4)
         d = exc1.asdict()
         self.assertEqual(d, {'node1.node2.3': 'exc1; exc2; exc3',
                              'node1.node4': 'exc1; exc4'})
@@ -87,13 +88,14 @@ class TestInvalid(unittest.TestCase):
         node2 = DummySchemaNode(Positional(), 'node2')
         node3 = DummySchemaNode(Positional(), 'node3')
         node4 = DummySchemaNode(Positional(), 'node4')
-        exc1 = self._makeOne(node1, 'exc1', pos=1)
-        exc2 = self._makeOne(node2, 'exc2', pos=2) 
-        exc3 = self._makeOne(node3, 'exc3', pos=3)
-        exc4 = self._makeOne(node4, 'exc4', pos=4)
-        exc1.add(exc2)
-        exc2.add(exc3)
-        exc1.add(exc4)
+        exc1 = self._makeOne(node1, 'exc1')
+        exc1.pos = 1
+        exc2 = self._makeOne(node2, 'exc2') 
+        exc3 = self._makeOne(node3, 'exc3')
+        exc4 = self._makeOne(node4, 'exc4')
+        exc1.add(exc2, 2)
+        exc2.add(exc3, 3)
+        exc1.add(exc4, 4)
         result = str(exc1)
         self.assertEqual(
             result,
@@ -135,10 +137,38 @@ class TestAll(unittest.TestCase):
         e = invalid_exc(validator, None, None)
         self.assertEqual(e.msg, ['msg1', 'msg2'])
 
+class TestFunction(unittest.TestCase):
+    def _makeOne(self, *arg, **kw):
+        from colander import Function
+        return Function(*arg, **kw)
+
+    def test_success_function_returns_True(self):
+        validator = self._makeOne(lambda x: True)
+        self.assertEqual(validator(None, None), None)
+
+    def test_fail_function_returns_empty_string(self):
+        validator = self._makeOne(lambda x: '')
+        e = invalid_exc(validator, None, None)
+        self.assertEqual(e.msg, 'Invalid value')
+
+    def test_fail_function_returns_False(self):
+        validator = self._makeOne(lambda x: False)
+        e = invalid_exc(validator, None, None)
+        self.assertEqual(e.msg, 'Invalid value')
+
+    def test_fail_function_returns_string(self):
+        validator = self._makeOne(lambda x: 'fail')
+        e = invalid_exc(validator, None, None)
+        self.assertEqual(e.msg, 'fail')
+
+    def test_propagation(self):
+        validator = self._makeOne(lambda x: 'a' in x, 'msg')
+        self.assertRaises(TypeError, validator, None, None)
+
 class TestRange(unittest.TestCase):
-    def _makeOne(self, min=None, max=None):
+    def _makeOne(self, **kw):
         from colander import Range
-        return Range(min=min, max=max)
+        return Range(**kw)
 
     def test_success_no_bounds(self):
         validator = self._makeOne()
@@ -161,11 +191,63 @@ class TestRange(unittest.TestCase):
         e = invalid_exc(validator, None, 0)
         self.assertEqual(e.msg, '0 is less than minimum value 1')
 
+    def test_min_failure_msg_override(self):
+        validator = self._makeOne(min=1, min_err='wrong')
+        e = invalid_exc(validator, None, 0)
+        self.assertEqual(e.msg, 'wrong')
+
     def test_max_failure(self):
         validator = self._makeOne(max=1)
         e = invalid_exc(validator, None, 2)
         self.assertEqual(e.msg, '2 is greater than maximum value 1')
 
+    def test_max_failure_msg_override(self):
+        validator = self._makeOne(max=1, max_err='wrong')
+        e = invalid_exc(validator, None, 2)
+        self.assertEqual(e.msg, 'wrong')
+
+class TestRegex(unittest.TestCase):
+    def _makeOne(self, pattern):
+        from colander import Regex
+        return Regex(pattern)
+        
+    def test_valid_regex(self):
+        self.assertEqual(self._makeOne('a')(None, 'a'), None)
+        self.assertEqual(self._makeOne('[0-9]+')(None, '1111'), None)
+        self.assertEqual(self._makeOne('')(None, ''), None)
+        self.assertEqual(self._makeOne('.*')(None, ''), None)
+
+    def test_invalid_regexs(self):
+        from colander import Invalid
+        self.assertRaises(Invalid, self._makeOne('[0-9]+'), None, 'a')
+        self.assertRaises(Invalid, self._makeOne('a{2,4}'), None, 'ba')
+
+class TestEmail(unittest.TestCase):
+    def _makeOne(self):
+        from colander import Email
+        return Email()
+
+    def test_valid_emails(self):
+        validator = self._makeOne()
+        self.assertEqual(validator(None, 'me@here.com'), None)
+        self.assertEqual(validator(None, 'me1@here1.com'), None)
+        self.assertEqual(validator(None, 'name@here1.us'), None)
+        self.assertEqual(validator(None, 'name@here1.info'), None)
+        self.assertEqual(validator(None, 'foo@bar.baz.biz'), None)
+
+    def test_empty_email(self):
+        validator = self._makeOne()
+        e = invalid_exc(validator, None, '')
+        self.assertEqual(e.msg, 'Invalid email address')
+      
+    def test_invalid_emails(self):
+        validator = self._makeOne()
+        from colander import Invalid
+        self.assertRaises(Invalid, validator, None, 'me@here.')
+        self.assertRaises(Invalid, validator, None, 'name@here.comcom')
+        self.assertRaises(Invalid, validator, None, '@here.us')
+        self.assertRaises(Invalid, validator, None, '(name)@here.info')
+
 class TestLength(unittest.TestCase):
     def _makeOne(self, min=None, max=None):
         from colander import Length
@@ -211,23 +293,6 @@ class TestOneOf(unittest.TestCase):
         e = invalid_exc(validator, None, None)
         self.assertEqual(e.msg, '"None" is not one of 1, 2')
 
-class TestType(unittest.TestCase):
-    def _makeOne(self):
-        from colander import Type
-        return Type()
-
-    def test_pserialize(self):
-        typ = self._makeOne()
-        typ.serialize = lambda *arg: arg
-        result = typ.pserialize(None, None)
-        self.assertEqual(result, (None, None))
-
-    def test_pdeserialize(self):
-        typ = self._makeOne()
-        typ.deserialize = lambda *arg: arg
-        result = typ.pdeserialize(None, None)
-        self.assertEqual(result, (None, None))
-
 class TestMapping(unittest.TestCase):
     def _makeOne(self, *arg, **kw):
         from colander import Mapping
@@ -339,101 +404,35 @@ class TestMapping(unittest.TestCase):
         result = typ.serialize(node, {'a':1})
         self.assertEqual(result, {'a':1})
 
-    def test_serialize_unknown_raise(self):
-        node = DummySchemaNode(None)
-        node.children = [DummySchemaNode(None, name='a')]
-        typ = self._makeOne('raise')
-        e = invalid_exc(typ.serialize, node, {'a':1, 'b':2})
-        self.assertEqual(e.msg, "Unrecognized keys in mapping: {'b': 2}")
-
-    def test_serialize_unknown_preserve(self):
-        node = DummySchemaNode(None)
-        node.children = [DummySchemaNode(None, name='a')]
-        typ = self._makeOne(unknown='preserve')
-        result = typ.serialize(node, {'a':1, 'b':2})
-        self.assertEqual(result, {'a':1, 'b':2})
-
-    def test_serialize_subnodes_raise(self):
-        node = DummySchemaNode(None)
-        node.children = [
-            DummySchemaNode(None, name='a', exc='Wrong 2'),
-            DummySchemaNode(None, name='b', exc='Wrong 2'),
-            ]
-        typ = self._makeOne()
-        e = invalid_exc(typ.serialize, node, {'a':1, 'b':2})
-        self.assertEqual(e.msg, None)
-        self.assertEqual(len(e.children), 2)
-
-    def test_serialize_subnode_missing_default(self):
-        node = DummySchemaNode(None)
-        node.children = [
-            DummySchemaNode(None, name='a'),
-            DummySchemaNode(None, name='b', default='abc'),
-            ]
-        typ = self._makeOne()
-        result = typ.serialize(node, {'a':1})
-        self.assertEqual(result, {'a':1, 'b':'abc'})
-
-    def test_serialize_subnode_missing_nodefault(self):
-        node = DummySchemaNode(None)
-        node.children = [
-            DummySchemaNode(None, name='a'),
-            DummySchemaNode(None, name='b'),
-            ]
-        typ = self._makeOne()
-        e = invalid_exc(typ.serialize, node, {'a':1})
-        self.assertEqual(e.children[0].msg, '"b" is required but missing')
-
     def test_serialize_subnode_partial(self):
         node = DummySchemaNode(None)
         node.children = [
             DummySchemaNode(None, name='a'),
             DummySchemaNode(None, name='b'),
             ]
-        typ = self._makeOne(partial=True)
+        typ = self._makeOne()
         result = typ.serialize(node, {'a':1})
         self.assertEqual(result, {'a':1})
 
-    def test_pserialize(self):
-        node = DummySchemaNode(None)
-        node.children = [
-            DummySchemaNode(None, name='a'),
-            DummySchemaNode(None, name='b'),
-            ]
-        typ = self._makeOne()
-        result = typ.pserialize(node, {'a':1})
-        self.assertEqual(result, {'a':1})
-
-    def test_pserialize_using_default(self):
+    def test_serialize_partial_with_default(self):
         node = DummySchemaNode(None)
         node.children = [
             DummySchemaNode(None, name='a'),
             DummySchemaNode(None, name='b', default='abc'),
             ]
         typ = self._makeOne()
-        result = typ.pserialize(node, {'a':1})
+        result = typ.serialize(node, {'a':1})
         self.assertEqual(result, {'a':1, 'b':'abc'})
 
-    def test_pdeserialize(self):
+    def test_serialize_with_unknown(self):
         node = DummySchemaNode(None)
         node.children = [
             DummySchemaNode(None, name='a'),
-            DummySchemaNode(None, name='b'),
             ]
         typ = self._makeOne()
-        result = typ.pdeserialize(node, {'a':1})
+        result = typ.serialize(node, {'a':1, 'b':2})
         self.assertEqual(result, {'a':1})
 
-    def test_pdeserialize_using_default(self):
-        node = DummySchemaNode(None)
-        node.children = [
-            DummySchemaNode(None, name='a'),
-            DummySchemaNode(None, name='b', default='abc'),
-            ]
-        typ = self._makeOne()
-        result = typ.pdeserialize(node, {'a':1})
-        self.assertEqual(result, {'a':1, 'b':'abc'})
-
 class TestTuple(unittest.TestCase):
     def _makeOne(self):
         from colander import Tuple
@@ -703,6 +702,14 @@ class TestString(unittest.TestCase):
         result = typ.serialize(node, uni)
         self.assertEqual(result, utf16)
 
+    def test_serialize_string_with_high_unresolveable_high_order_chars(self):
+        not_utf8 = '\xff\xfe\xf8\x00'
+        node = DummySchemaNode(None)
+        typ = self._makeOne('utf-8')
+        e = invalid_exc(typ.serialize, node, not_utf8)
+        self.failUnless('cannot be serialized to str' in e.msg)
+        
+
 class TestInteger(unittest.TestCase):
     def _makeOne(self):
         from colander import Integer
@@ -1118,6 +1125,20 @@ class TestDate(unittest.TestCase):
         e = invalid_exc(typ.deserialize, node, '10-10-10-10')
         self.failUnless('cannot be parsed' in e.msg)
 
+    def test_deserialize_empty_required(self):
+        node = DummySchemaNode(None)
+        typ = self._makeOne()
+        e = invalid_exc(typ.deserialize, node, '')
+        self.failUnless('Required' in e.msg)
+
+    def test_deserialize_empty_notrequired(self):
+        node = DummySchemaNode(None)
+        node.default = 'abc'
+        node.required = False
+        typ = self._makeOne()
+        result = typ.deserialize(node, None)
+        self.assertEqual(result, 'abc')
+
     def test_deserialize_success_date(self):
         import datetime
         typ = self._makeOne()
@@ -1199,24 +1220,12 @@ class TestSchemaNode(unittest.TestCase):
         e = invalid_exc(node.deserialize, 1)
         self.assertEqual(e.msg, 'Wrong')
 
-    def test_pdeserialize(self):
-        typ = DummyType()
-        node = self._makeOne(typ)
-        result = node.pdeserialize(1)
-        self.assertEqual(result, 1)
-
     def test_serialize(self):
         typ = DummyType()
         node = self._makeOne(typ)
         result = node.serialize(1)
         self.assertEqual(result, 1)
 
-    def test_pserialize(self):
-        typ = DummyType()
-        node = self._makeOne(typ)
-        result = node.pserialize(1)
-        self.assertEqual(result, 1)
-
     def test_add(self):
         node = self._makeOne(None)
         node.add(1)
@@ -1277,9 +1286,9 @@ class TestSchema(unittest.TestCase):
         self.assertEqual(node.children[0].typ.__class__, colander.String) 
         self.assertEqual(node.children[0].title, 'Thing')
         self.assertEqual(node.children[1].title, 'bar')
-        
+
 class TestSequenceSchema(unittest.TestCase):
-    def test_it(self):
+    def test_succeed(self):
         import colander
         _inner = colander.SchemaNode(colander.String())
         class MySchema(colander.SequenceSchema):
@@ -1290,6 +1299,27 @@ class TestSequenceSchema(unittest.TestCase):
         self.assertEqual(node.typ.__class__, colander.Sequence)
         self.assertEqual(node.children[0], _inner)
 
+    def test_fail_toomany(self):
+        import colander
+        thingnode = colander.SchemaNode(colander.String())
+        thingnode2 = colander.SchemaNode(colander.String())
+        class MySchema(colander.SequenceSchema):
+            thing = thingnode
+            thing2 = thingnode2
+        e = invalid_exc(MySchema)
+        self.assertEqual(
+            e.msg,
+            'Sequence schemas must have exactly one child node')
+
+    def test_fail_toofew(self):
+        import colander
+        class MySchema(colander.SequenceSchema):
+            pass
+        e = invalid_exc(MySchema)
+        self.assertEqual(
+            e.msg,
+            'Sequence schemas must have exactly one child node')
+
 class TestTupleSchema(unittest.TestCase):
     def test_it(self):
         import colander
@@ -1463,9 +1493,6 @@ class DummySchemaNode(object):
             raise Invalid(self, self.exc)
         return val
 
-    pserialize = serialize
-    pdeserialize = deserialize
-
 class DummyValidator(object):
     def __init__(self, msg=None):
         self.msg = msg
@@ -1488,9 +1515,3 @@ class DummyType(object):
     def deserialize(self, node, value):
         return value
 
-    def pserialize(self, node, value):
-        return value
-    
-    def pdeserialize(self, node, value):
-        return value
-
